package meerkat.bulletinboard.workers;

import meerkat.bulletinboard.AsyncBulletinBoardClient.ClientCallback;
import meerkat.bulletinboard.MultiServerWorker;
import meerkat.bulletinboard.SingleServerBulletinBoardClient;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.BulletinBoardAPI.*;

import java.util.Iterator;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by Arbel Deutsch Peled on 27-Dec-15.
 */
public class MultiServerGetRedundancyWorker extends MultiServerWorker<MessageID, Float> {

    private AtomicInteger serversContainingMessage;
    private AtomicInteger totalContactedServers;

    public MultiServerGetRedundancyWorker(List<SingleServerBulletinBoardClient> clients,
                                         int minServers, MessageID payload, int maxRetry,
                                         ClientCallback<Float> clientCallback) {

        super(clients, minServers, payload, maxRetry, clientCallback); // Shuffle clients on creation to balance load

        serversContainingMessage = new AtomicInteger(0);
        totalContactedServers = new AtomicInteger(0);

    }

    /**
     * This method carries out the actual communication with the servers via HTTP Post
     * It accesses the servers in a random order until one answers it
     * Successful retrieval from any server terminates the method and returns the received values; The list is not changed
     * @return The original job and the list of messages found in the first server that answered the query
     * @throws CommunicationException
     */
    public void run(){

        Iterator<SingleServerBulletinBoardClient> clientIterator = getClientIterator();

        // Iterate through clients

        while (clientIterator.hasNext()) {

            SingleServerBulletinBoardClient client = clientIterator.next();

            // Send request to client
            client.getRedundancy(payload,this);

        }

    }

    @Override
    public void handleCallback(Float result) {

        if (result > 0.5) {
            serversContainingMessage.incrementAndGet();
        }

        if (totalContactedServers.incrementAndGet() >= getClientNumber()){
            succeed(new Float(((float) serversContainingMessage.get()) / ((float) getClientNumber()) ));
        }

    }

    @Override
    public void handleFailure(Throwable t) {
       handleCallback(new Float(0.0));
    }

}
