package meerkat.bulletinboard.workers;

import meerkat.bulletinboard.AsyncBulletinBoardClient.ClientCallback;
import meerkat.bulletinboard.MultiServerWorker;
import meerkat.bulletinboard.SingleServerBulletinBoardClient;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.BulletinBoardAPI.*;

import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;
import java.util.Iterator;
import java.util.List;

/**
 * Created by Arbel Deutsch Peled on 27-Dec-15.
 */
public class MultiServerPostMessageWorker extends MultiServerWorker<BulletinBoardMessage, Boolean> {

    public MultiServerPostMessageWorker(List<SingleServerBulletinBoardClient> clients,
                                        int minServers, BulletinBoardMessage payload, int maxRetry,
                                        ClientCallback<Boolean> clientCallback) {

        super(clients, minServers, payload, maxRetry, clientCallback);

    }

    /**
     * This method carries out the actual communication with the servers via HTTP Post
     * It accesses the servers one by one and tries to post the payload to each in turn
     * The method will only iterate once through the server list
     * Successful post to a server results in removing the server from the list
     * @return The original job, but with a modified server list
     * @throws CommunicationException
     */
    public void run() {

        WebTarget webTarget;
        Response response;

        int count = 0; // Used to count number of servers which contain the required message in a GET_REDUNDANCY request.

        // Iterate through servers

        Iterator<SingleServerBulletinBoardClient> clientIterator = getClientIterator();

        while (clientIterator.hasNext()) {

            // Send request to Server
            SingleServerBulletinBoardClient client = clientIterator.next();

            client.postMessage(payload, this);

        }

    }

    @Override
    public void handleCallback(Boolean result) {
        if (result){
            if (minServers.decrementAndGet() <= 0){
                succeed(Boolean.TRUE);
            }
        }
    }

    @Override
    public void handleFailure(Throwable t) {
        if (maxFailedServers.decrementAndGet() < 0){
            fail(t);
        }
    }
}
