package meerkat.bulletinboard.workers;

import meerkat.bulletinboard.AsyncBulletinBoardClient.ClientCallback;
import meerkat.bulletinboard.MultiServerWorker;
import meerkat.bulletinboard.SingleServerBulletinBoardClient;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.BulletinBoardAPI.*;

import java.util.Iterator;
import java.util.List;


/**
 * Created by Arbel Deutsch Peled on 27-Dec-15.
 */
public class MultiServerReadMessagesWorker extends MultiServerWorker<MessageFilterList,List<BulletinBoardMessage>>{

    private Iterator<SingleServerBulletinBoardClient> clientIterator;

    public MultiServerReadMessagesWorker(List<SingleServerBulletinBoardClient> clients,
                                        int minServers, MessageFilterList payload, int maxRetry,
                                        ClientCallback<List<BulletinBoardMessage>> clientCallback) {

        super(clients, true, minServers, payload, maxRetry, clientCallback); // Shuffle clients on creation to balance load

        clientIterator = getClientIterator();

    }

    /**
     * This method carries out the actual communication with the servers via HTTP Post
     * It accesses the servers in a random order until one answers it
     * Successful retrieval from any server terminates the method and returns the received values; The list is not changed
     * @return The original job and the list of messages found in the first server that answered the query
     * @throws CommunicationException
     */
    public void run(){

        // Iterate through servers

        if (clientIterator.hasNext()) {

            // Send request to Server
            SingleServerBulletinBoardClient client = clientIterator.next();

            // Retrieve answer
            client.readMessages(payload,this);

        } else {
            fail(new CommunicationException("Could not contact any server"));
        }

    }

    @Override
    public void handleCallback(List<BulletinBoardMessage> msg) {
        succeed(msg);
    }

    @Override
    public void handleFailure(Throwable t) {
        run(); // Retry with next server
    }

}
