package meerkat.bulletinboard.workers;

import meerkat.bulletinboard.SingleServerWorker;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.BulletinBoardAPI.BoolMsg;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.rest.Constants;

import javax.ws.rs.ProcessingException;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

import static meerkat.bulletinboard.BulletinBoardConstants.BULLETIN_BOARD_SERVER_PATH;
import static meerkat.bulletinboard.BulletinBoardConstants.POST_MESSAGE_PATH;

/**
 * Created by Arbel Deutsch Peled on 27-Dec-15.
 * Tries to contact server once and perform a post operation
 */
public class SingleServerPostMessageWorker extends SingleServerWorker<BulletinBoardMessage, Boolean> {

    public SingleServerPostMessageWorker(String serverAddress, BulletinBoardMessage payload, int maxRetry) {
        super(serverAddress, payload, maxRetry);
    }

    /**
     * This method carries out the actual communication with the server via HTTP Post
     * It accesses the server and tries to post the payload to it
     * Successful post to a server results
     * @return TRUE if the operation is successful
     * @throws CommunicationException if the operation is unseccessful
     */
    public Boolean call() throws CommunicationException{

        Client client = clientLocal.get();

        WebTarget webTarget = client.target(serverAddress).path(BULLETIN_BOARD_SERVER_PATH).path(POST_MESSAGE_PATH);;
        Response response = webTarget.request(Constants.MEDIATYPE_PROTOBUF).post(
                Entity.entity(payload, Constants.MEDIATYPE_PROTOBUF));

        try {

            // If a BoolMsg entity is returned: the post was successful
            response.readEntity(BoolMsg.class);
            return Boolean.TRUE;

        } catch (ProcessingException | IllegalStateException e) {

            // Post to this server failed
            throw new CommunicationException("Could not contact the server");

        }
        finally {
            response.close();
        }

    }

}
